; $Id: dcs_read_binaryoctavefile.pro,v 1.7 2002/07/13 20:14:57 john Exp $
; Written by J.R.D. Copley.
;************************************************************************************************
pro dcs_read_binaryOctavefile_done_reading,unit,ivar,ivaru,variables_read,unknowns,$
	davePtr=davePtr
;************************************************************************************************	;
;
compile_opt strictarr
;
if (ivar ge 0) then variables_read=variables_read[0:ivar] else variables_read=["None"]
if (ivaru ge 0) then unknowns=unknowns[0:ivaru] else begin
	unknowns=["None"]
endelse
;
free_lun,unit
;
; Populate (*histPtr).x and (*histPtr).y.
ntchan=(*(*(*davePtr).dataStrPtr).specificPtr).ntchan
nhedet=(*(*(*davePtr).dataStrPtr).specificPtr).nhedet
(*(*(*davePtr).dataStrPtr).commonStr.histPtr).x=indgen(ntchan)+1
(*(*(*davePtr).dataStrPtr).commonStr.histPtr).y=indgen(nhedet)+1
;
; Initialize normalization_sum and fbnormalization_sum.
(*(*(*davePtr).dataStrPtr).specificPtr).normalization_sum=fltarr(nhedet)
(*(*(*davePtr).dataStrPtr).specificPtr).fbnormalization_sum=fltarr(nhedet)
;
; Define time channel boundaries in units of time sample-detector.
twid=(*(*(*davePtr).dataStrPtr).specificPtr).tchanlook[0,*]*0.05
tsdmin=(*(*(*davePtr).dataStrPtr).specificPtr).tsdmin
times=tsdmin+[0,reform(total(twid,/cumulative))]
free_ptr,(*(*(*davePtr).dataStrPtr).specificPtr).time_propsPtr
(*(*(*davePtr).dataStrPtr).specificPtr).time_propsPtr=ptr_new(times)
;
; Define time channel boundaries in units of energy transfer.
conversion=!dcs_hsq2mn/(!dcs_hom/!dcs_dsd)^2
wl0=(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl
energies=!dcs_hsq2mn/wl0^2-conversion/times^2
free_ptr,(*(*(*davePtr).dataStrPtr).specificPtr).energy_propsPtr
(*(*(*davePtr).dataStrPtr).specificPtr).energy_propsPtr=ptr_new(energies)
;
; Define detector angles (as a vector).
dcs_getangles,angles,nhedet=nhedet
free_ptr,(*(*(*davePtr).dataStrPtr).specificPtr).angle_propsPtr
(*(*(*davePtr).dataStrPtr).specificPtr).angle_propsPtr=ptr_new(angles)
;
; Define elastic Q values (as a vector).
free_ptr,(*(*(*davePtr).dataStrPtr).specificPtr).Q_propsPtr
Qs=4.0*!dpi*sin(0.5*angles*!dtor)/(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl
(*(*(*davePtr).dataStrPtr).specificPtr).Q_propsPtr=ptr_new(Qs)
;
return
end


;************************************************************************************************
pro dcs_read_binaryOctavefile,filepath,nofirstline,omithisto,variables_read,unknowns,error,$
	davePtr=davePtr
;************************************************************************************************
;
compile_opt strictarr
;
; This is a procedure that reads a binary Octave file.
; If omithisto=0, nothing is omitted, everything is read.
; If omithisto=1, histohigh is read but histodata is omitted.
; If omithisto=2, histohigh and histodata are both omitted.
;
on_ioerror, no_file
;
; Determine whether or not file is compressed.
ending=strmid(filepath,2,3,/reverse_offset)
compressed=(ending eq ".gz")
;
; Read header to determine byte ordering ("endianness")
openr,unit,filepath,compress=compressed,/get_lun
header=bytarr(11)
readu,unit,header
free_lun,unit
;
case string(header[9]) of
	"B": openr,unit,filepath,/swap_endian,compress=compressed,/get_lun
	"L": openr,unit,filepath,compress=compressed,/get_lun
	else: begin
		error="Cannot determine byte ordering of file named "+filepath
		return
	end
endcase
;
readu,unit,header
;
; Define the types of some variables
nlgth=0l & dldocgf=bytarr(5) & dt=0b & padding=0b & scalar=0.0d
n1=0l & n2=0l & nstrgs=0l & nchars=0l
;
variables_read=strarr(100)
unknowns=strarr(100)
error=""
ivar=-1
ivaru=-1
;
ptr_free,(*(*davePtr).dataStrPtr).commonStr.histPtr
ntchan=(*(*(*davePtr).dataStrPtr).specificPtr).ntchan
nhedet=(*(*(*davePtr).dataStrPtr).specificPtr).nhedet
(*(*davePtr).dataStrPtr).commonStr.histPtr = $
	ptr_new({qty:fltarr(ntchan,nhedet),err:fltarr(ntchan,nhedet),$
	x:fltarr(ntchan),y:fltarr(nhedet)})
;
ptr_free,(*(*davePtr).dataStrPtr).commonStr.treatmentPtr
(*(*davePtr).dataStrPtr).commonStr.treatmentPtr = ptr_new("")
;
while (not eof(unit)) do begin
	readu,unit,nlgth; read name_length
	name=bytarr(nlgth) & readu,unit,name & name=string(name); read name
	if ((omithisto eq 2 and name eq "histohigh") or $
		(omithisto eq 1 and name eq "histodata")) then begin
		dcs_read_binaryOctavefile_done_reading,unit,ivar,ivaru,$
			variables_read,unknowns,davePtr=davePtr
		return
	endif
	readu,unit,dldocgf; read 5 bytes of doc_length, doc, and global flag
	readu,unit,dt; read data type
	case dt of
		1: begin; read scalar
			readu,unit,padding
			readu,unit,scalar
				case name of
					"badbrd": (*(*(*davePtr).dataStrPtr).specificPtr).badbrd=float(scalar)
					"baddet": (*(*(*davePtr).dataStrPtr).specificPtr).baddet=float(scalar)
					"ch_ms": (*(*(*davePtr).dataStrPtr).specificPtr).ch_ms=float(scalar)
					"ch_res": (*(*(*davePtr).dataStrPtr).specificPtr).ch_res=fix(scalar)
					"ch_srdenom": (*(*(*davePtr).dataStrPtr).specificPtr).ch_srdenom=fix(scalar)
					"ch_srmode": (*(*(*davePtr).dataStrPtr).specificPtr).ch_srmode=fix(scalar)
					"ch_wl": (*(*(*davePtr).dataStrPtr).specificPtr).ch_wl=float(scalar)
					"coll_amp": (*(*(*davePtr).dataStrPtr).specificPtr).coll_amp=float(scalar)
					"coll_mean": (*(*(*davePtr).dataStrPtr).specificPtr).coll_mean=float(scalar)
					"coll_osc": (*(*(*davePtr).dataStrPtr).specificPtr).coll_osc=fix(scalar)
					"datamax": (*(*(*davePtr).dataStrPtr).specificPtr).datamax=float(scalar)
					"det_dis": (*(*(*davePtr).dataStrPtr).specificPtr).det_dis=float(scalar)
					"duration": (*(*(*davePtr).dataStrPtr).specificPtr).duration=float(scalar)
					"highmax": (*(*(*davePtr).dataStrPtr).specificPtr).highmax=float(scalar)
					"nframes": (*(*(*davePtr).dataStrPtr).specificPtr).nframes=long(scalar)
					"temp_setpoint": (*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint=float(scalar)
					"tsdmin": (*(*(*davePtr).dataStrPtr).specificPtr).tsdmin=float(scalar)
					else: begin & ivaru=ivaru+1 & unknowns[ivaru]=name & end
				endcase
				ivar=ivar+1
				variables_read[ivar]=name
		end
		2: begin; read matrix
			readu,unit,n1,n2
			nhi=n1>n2
			nlo=n1<n2
			if (nlo eq 1) then data=dblarr(nhi) else data=dblarr(n1,n2)
			readu,unit,padding
			readu,unit,data
				case name of
					"ch_bid" : (*(*(*davePtr).dataStrPtr).specificPtr).ch_bid=float(data)
					"ch_delay": (*(*(*davePtr).dataStrPtr).specificPtr).ch_delay=float(data)
					"ch_dis": (*(*(*davePtr).dataStrPtr).specificPtr).ch_dis=float(data)
					"ch_input": (*(*(*davePtr).dataStrPtr).specificPtr).ch_input=float(data)
					"ch_phase": (*(*(*davePtr).dataStrPtr).specificPtr).ch_phase=float(data)
					"ch_slots": (*(*(*davePtr).dataStrPtr).specificPtr).ch_slots=float(data)
					"detsum": (*(*(*davePtr).dataStrPtr).specificPtr).detsum=ulong(data)
					"fc_dis": (*(*(*davePtr).dataStrPtr).specificPtr).fc_dis=float(data)
					"grandsum": (*(*(*davePtr).dataStrPtr).specificPtr).grandsum=ulong(data)
					"histodata": begin
						(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty=ulong(data) ; CHANGED 4/15/02
						(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err=sqrt(ulong(data)) ; CHANGED 4/15/02
					end
					"histohigh": (*(*(*davePtr).dataStrPtr).specificPtr).histohigh=ulong(data) ; CHANGED 4/15/02
					"motor_pos": (*(*(*davePtr).dataStrPtr).specificPtr).motor_pos=float(data)
					"repeats": (*(*(*davePtr).dataStrPtr).specificPtr).repeats=ulong(data)
					"resets": (*(*(*davePtr).dataStrPtr).specificPtr).resets=ulong(data)
					"tchanlook": (*(*(*davePtr).dataStrPtr).specificPtr).tchanlook=ulong(data)
					"temp_control": begin
						(*(*(*davePtr).dataStrPtr).specificPtr).temp_control=-1.1
						(*(*(*davePtr).dataStrPtr).specificPtr).temp_control=float(data)
					end
					"temp_sample": begin
						(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample=-1.1
						(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample=float(data)
					end
					"timsum": (*(*(*davePtr).dataStrPtr).specificPtr).timsum=ulong(data)
					"totals": (*(*(*davePtr).dataStrPtr).specificPtr).totals=ulong(data)
					else: begin & ivaru=ivaru+1 & unknowns[ivaru]=name & end
				endcase
				ivar=ivar+1
				variables_read[ivar]=name
		end
		5: begin; read string
			readu,unit,nchars; read number of characters in string element
			if (nchars gt 0) then begin
				str=bytarr(nchars) & readu,unit,str; read string element
				str=string(str)
				case name of
					"command": (*(*(*davePtr).dataStrPtr).specificPtr).command=str
					"comments": (*(*(*davePtr).dataStrPtr).specificPtr).comments=str
					"sample_desc": (*(*(*davePtr).dataStrPtr).specificPtr).sample_desc=str
					"start_date": (*(*(*davePtr).dataStrPtr).specificPtr).start_date=str
					"startchoice": (*(*(*davePtr).dataStrPtr).specificPtr).startchoice=str
					"stop_date": (*(*(*davePtr).dataStrPtr).specificPtr).stop_date=str
					else: begin & ivaru=ivaru+1 & unknowns[ivaru]=name & end
				endcase
				ivar=ivar+1
				variables_read[ivar]=name
			endif
		end
		7: begin; read string array
			readu,unit,nstrgs; read number of strings in array
			strarray=strarr(nstrgs)
			for j=1,nstrgs do begin
				readu,unit,nchars; read number of characters in string element
				str=bytarr(nchars) & readu,unit,str; read string element
				strarray[j-1]=strjoin(string(str)); store, joined, in strarray
			endfor
			case name of
				"highsource": (*(*(*davePtr).dataStrPtr).specificPtr).highsource=strarray
				else: begin & ivaru=ivaru+1 & unknowns[ivaru]=name & end
			endcase
			ivar=ivar+1
			variables_read[ivar]=name
		end
		else: error="N.B. An unsupported variable type has been encountered: "+strcompress(dt)
	endcase
endwhile
;
dcs_read_binaryOctavefile_done_reading,unit,ivar,ivaru,$
	variables_read,unknowns,davePtr=davePtr
return
;
no_file: free_lun,unit
	error="There is no file named "+filepath
end