; $Id: fans_changeviewdata.pro,v 1.5 2002/07/16 16:09:31 azuah Exp $
;===============================================================================
; Event Handlers for FANS_DATREDUC program defined in fans_datreduc.pro.
; Richard Tumanjong Azuah
;
; May 28 2002 - Initial writing
; 
;===============================================================================


;===============================================================================
; Event handler for the Fit par text widget ie when new Fast bkgd
; parameters are entered manually
pro fans_fastPar, event

widget_control, event.top, get_uvalue=sPtr

; retrieve parameters from the text widget
widget_control, event.id, get_value=parstr
if (n_elements(parstr) le 0) then return
fpar = [0.0,0.0,0.0]
tokens = strsplit(parstr[0],' ,',/extract) ; tokenize the string
for i=0,n_elements(tokens)-1 do begin ; convert each token into a float
    reads, tokens[i], af
    fpar[i] = af
endfor 

; generate curve from input poly parameters - first get energy array
index = (*sPtr).fastIndex
fdStructPtr = (*(*sPtr).fDataPtr)[index] ; get a ptr to the approp. fans data struct
omega = (*(*fdStructPtr).energyPtr) ; energy tranfer

; if detectors are to be masked, this has to be taken into account
; when fitting the fast bkgd data
reset = 0
widget_control, (*sPtr).mskOnID, get_uvalue=mskon
if (mskon) then begin 
    goodDets = -1
    goodDets = fans_maskDets(sPtr) ; get a list of good dets
    if ((ptr_valid((*sPtr).mskListPtr) eq 1) or goodDets[0] ne -1) then begin
        reset = 1               ; to make sure data reverts to orig cnts afterwards
        (*sPtr).updFast = 0     ; records that fast fit is in sync with mask dets
        nspec = (*fastDataPtr).nspec
        origTotCnts = (*(*fastDataPtr).totCntsPtr)   
        for i = 0,nspec-1 do begin  ; re-calculate total cnts using only good dets
            (*(*fastDataPtr).totCntsPtr)[i] = total((*(*fastDataPtr).detCntsPtr)[goodDets,i])
            (*(*fastDataPtr).totErrsPtr)[i] = sqrt(total(((*(*fastDataPtr).detErrsPtr)[goodDets,i])^2))
        endfor
    endif
endif

yfit = fpar[0] + fpar[1]*omega + fpar[2]*(omega^2)

fans_plotData, sPtr, 'fast', err=err,emsg=emsg ; plot fast bkgd

oplot, omega,yfit,linestyle=2,thick=4 ; overplot polynomial fit 

; reset output calculation flag
(*sPtr).doOutpCalc = 1

; make 'sub FFit' widget active
widget_control, (*sPtr).subFastID, sensitive=1

; reset original total cnts if necessary
if (reset) then (*(*fastDataPtr).totCntsPtr) =  origTotCnts

end


;===============================================================================
; Event handler for the Fit Fast button
pro fans_Fastfit, event

widget_control, event.top, get_uvalue=sPtr

index = (*sPtr).fastIndex
fastDataPtr = (*(*sPtr).fDataPtr)[index] ; get a ptr to fast data
omega = (*(*fastDataPtr).energyPtr) ; energy tranfers

; if detectors are to be masked, this has to be taken into account
; when fitting the fast bkgd data
reset = 0
widget_control, (*sPtr).mskOnID, get_uvalue=mskon
if (mskon) then begin 
    goodDets = -1
    goodDets = fans_maskDets(sPtr) ; get a list of good dets
    if ((ptr_valid((*sPtr).mskListPtr) eq 1) or goodDets[0] ne -1) then begin
        reset = 1               ; to make sure data reverts to orig cnts afterwards
        (*sPtr).updFast = 0     ; records that fast fit is in sync with mask dets
        nspec = (*fastDataPtr).nspec
        origTotCnts = (*(*fastDataPtr).totCntsPtr)   
        for i = 0,nspec-1 do begin ; re-calculate total cnts using only good dets
            (*(*fastDataPtr).totCntsPtr)[i] = total((*(*fastDataPtr).detCntsPtr)[goodDets,i])
            (*(*fastDataPtr).totErrsPtr)[i] = sqrt(total(((*(*fastDataPtr).detErrsPtr)[goodDets,i])^2))
        endfor
    endif
endif

; fit a polynomial of degree 2
totCnts = (*(*fastDataPtr).totCntsPtr)
par = poly_fit(omega,totCnts,2,measure_error=sqrt(totCnts))

; display fit results in fitted par widget
mval = max(par)
fstr = (mval gt 100.0) ? '(F6.2)' : '(F5.2)'
parstr = ''
for i = 0,2 do begin
    buf = strtrim(string(format=fstr,par[i]),2)
    parstr = parstr+buf+' '
endfor
widget_control, (*sPtr).parID, set_value=strtrim(parstr,2)

; generate curve from fitted parameters
yfit = par[0] + par[1]*omega + par[2]*(omega^2)

fans_plotData, sPtr, 'fast', err=err,emsg=emsg ; plot fast bkgd
oplot, omega,yfit,linestyle=2,thick=4 ; overplot fitted polynomial

; make 'sub FFit' widget active
widget_control, (*sPtr).subFastID, sensitive=1

; switch on output calculation flag
(*sPtr).doOutpCalc = 1

; reset original total cnts if necessary
if (reset) then (*(*fastDataPtr).totCntsPtr) =  origTotCnts

end




;===============================================================================
; Deals with events triggered when there is a change in the type
; of data to be displayed/plotted
pro fans_changeViewData, event

widget_control, event.top, get_uvalue=sPtr

dataType = widget_info(event.id,/uname)

; radio buttons fire an event when they are selected/deselected and
; both will result in a call to fans_changeDispType. When a button is
; deslected/selected, its uvalue has to be changed to 0/1 respectively.
switch event.select of
    0: begin                    ; (exclusive) radio button deselected
        switch dataType of

            'samp': begin
                widget_control, (*sPtr).sViewID, set_uvalue=0
                widget_control, (*sPtr).plotWinID, sensitive=0
                break
            end

            'bkgd': begin
                widget_control, (*sPtr).bViewID, set_uvalue=0
                widget_control, (*sPtr).plotWinID, sensitive=0
                break
            end

            'fast': begin
                ;; switch off some functionality
                widget_control, (*sPtr).fViewID, set_uvalue=0
                widget_control, (*sPtr).fitBkgdID, sensitive=0
                widget_control, (*sPtr).fitLabID, sensitive=0
                widget_control, (*sPtr).parID, sensitive=0
                break
            end

            'outp': begin
                widget_control, (*sPtr).oViewID, set_uvalue=0
                break
            end

        endswitch

        break
    end
    
    1: begin                    ; (exclusive) radio button selected
        switch dataType of
            
            'samp': begin
                
                widget_control, (*sPtr).sViewID, set_uvalue=1
                widget_control, (*sPtr).plotWinID, sensitive=1
                widget_control, (*sPtr).eSliderID, sensitive=1
                ;; Update current data
                index = (*sPtr).sampIndex
                dataPtr = (*(*sPtr).sDataPtr)[index]
                
                ;; Update plot and slider using selected data
                updatePlotSlider, sPtr,dataType,dataPtr,err=err,emsg=emsg
                break
            end

            'bkgd': begin
                widget_control, (*sPtr).bViewID, set_uvalue=1
                widget_control, (*sPtr).plotWinID, sensitive=1
                widget_control, (*sPtr).eSliderID, sensitive=1
                ;; Update current data
                index = (*sPtr).bkgdIndex
                dataPtr = (*(*sPtr).bDataPtr)[index]

                ;; Update plot and slider using selected data
                updatePlotSlider, sPtr,dataType,dataPtr,err=err,emsg=emsg

                break
            end

            'fast': begin
                ;; switch on some functionality specific to Fast Bkgd
                widget_control, (*sPtr).fViewID, set_uvalue=1
                widget_control, (*sPtr).fitBkgdID, sensitive=1
                widget_control, (*sPtr).fitLabID, sensitive=1
                widget_control, (*sPtr).parID, sensitive=1
                widget_control, (*sPtr).eSliderID, sensitive=0

                ;; update plot
                fans_plotData, sPtr, 'fast', err=err,emsg=emsg

                break
            end

            'outp': begin
                widget_control, (*sPtr).oViewID, set_uvalue=1
                widget_control, (*sPtr).eSliderID, sensitive=0
                ;; Calculate output data
                fans_outpCalc, sPtr, err=err, emsg=emsg
                if (err ne 0) then begin
                    printerror,err,emsg
                    return
                endif

                ;; update plot
                fans_plotData, sPtr, 'outp', err=err,emsg=emsg

                break
            end

        endswitch


        
        break
    end

endswitch                       ; switch event.select
    
end



