; $Id: fans_datreduc.pro,v 1.7 2002/07/16 16:09:31 azuah Exp $
;===============================================================================
; FANS_DATREDUC
; FANS data redunction module. It performs the following tasks
;  ** Load one or more sample files
;  ** Load one or more background files
;  ** Load one or more Fast bkgd files
;  ** Allows user to mask (remove) detectors after visual inspection
;     of data
;  ** Correct data for fast bkgd (using a polynomial fit to it)
;  ** Perform bkgd subtraction if required.
;
; Richard Tumanjong Azuah
;
; May 14 2002 -- Initial writing
;
;===============================================================================



;===============================================================================
; dummy event handler
;
pro dummy, event
if (!debug) then begin
    print,'Dummy handler called...'

    help, event,/structure

    widget_control, event.top, get_uvalue=sPtr
    
    widget_control, event.id, get_value=value
    widget_control, event.id, get_uvalue=uvalue
    uname = widget_info(event.id,/uname)
    print,'DUMMY: Value = ',value,'  **Uname = ',uname ;,'  **Uvalue = ',uvalue
endif

end


;===============================================================================
pro fans_datReduc_cleanup, tlbID

;print, 'Cleaning up...'
widget_control, tlbID, get_uvalue=sPtr

if (!debug) then print,'Validity of sPtr is ',ptr_valid(sPtr)

;---- BEGIN DAVE specific cleanup section
fansInfo = {FANSDATREDUCEVENT,$
          id:(*sPtr).notifyIDs[0],$
          top:(*sPtr).notifyIDs[1],$
          handler:0L}
if widget_info((*sPtr).notifyIDs[0],/valid_id) then begin
  widget_control,(*sPtr).notifyIDs[0],send_event = fansInfo
endif
;---- END DAVE specific section

; delete all data currently in memory
if (ptr_valid((*sPtr).sDataPtr)) then $
  fans_cleanDataPtrs, sPtr,'samp' ; sample data
if (ptr_valid((*sPtr).bDataPtr)) then $
  fans_cleanDataPtrs, sPtr,'bkgd' ; background data
if (ptr_valid((*sPtr).fDataPtr)) then $
  fans_cleanDataPtrs, sPtr,'fast' ; fast background data
if (ptr_valid((*sPtr).oDataPtr)) then $
  fans_cleanDataPtrs, sPtr,'outp' ; output data

; deal with the remainder of the state pointer
ptr_free, (*sPtr).mfoPtr, $
          (*sPtr).sampListPtr, $
          (*sPtr).bkgdListPtr, $
          (*sPtr).fastListPtr, $
          (*sPtr).outpListPtr, $
          (*sPtr).mskListPtr, $
          sPtr

if (!debug) then print,'Validity of sPtr is ',ptr_valid(sPtr)

end


;===============================================================================
pro fans_datReduc, davePtr,notifyIds=notifyIds,group_leader=group_leader

compile_opt idl2

; define container base widgets
tlbID = widget_base(group_leader=group_leader,/col,title='FANS DATA REDUCTION',mbar=mbar)

;cb1ID = widget_base(tlbID,/row,frame=1)
;cb1aID = widget_base(cb1ID,/base_align_left)
;cb1bID = widget_base(cb1ID,/row,/exclusive)

cb2ID = widget_base(tlbID,/row,frame=1)
;cb2aID = widget_base(cb2ID,/row)
;cb2bID = widget_base(cb2ID,/row)

cb3ID = widget_base(tlbID,/col,frame=1)
cb3aID = widget_base(cb3ID,/row)
cb3bID = widget_base(cb3aID,/row)
cb3cID = widget_base(cb3aID,/row,/exclusive,event_pro='fans_changeviewdata')
cb3dID = widget_base(cb3aID,/row,frame=1)

cb4ID = widget_base(tlbID,/col,frame=1,event_pro='fans_sliderEvent')
cb4aID = widget_base(cb4ID,/row)

cb5ID = widget_base(tlbID,/row,frame=1,/align_center)
cb5aID = widget_base(cb5ID,/row)
cb5bID = widget_base(cb5ID,/row,/nonexclusive)
;cb5cID = widget_base(cb5ID,/row)
;cb5dID = widget_base(cb5ID,/row,frame=1,/exclusive,event_pro='dummy')

;cb6ID = widget_base(tlbID,/row)

; bulid the menus. There are three drop-down menus: file, settings and help
fileMenu = widget_button(mbar, value='File',/menu, $
                         event_pro='fans_filemenu')
settingsMenu = widget_button(mbar, value='Settings',/menu, $
                         event_pro='fans_settings')
helpMenu = widget_button(mbar, value='Help',/menu, $
                         event_pro='dummy')

; fill the 'File' drop-down menu
dirMID = widget_button(fileMenu,value='Change Data Directory',uname='dir')
sampMID = widget_button(fileMenu,value='Select Sample File(s)',uname='samp')
bkgdMID = widget_button(fileMenu,value='Select Bkgd File(s)',uname='bkgd')
fbkgdMID = widget_button(fileMenu,value='Select Fast Bkgd File(s)',uname='fast')
save1MID = widget_button(fileMenu,value='Save (DAVE) File',uname='sdave',/separator)
save2MID = widget_button(fileMenu,value='Save (ASCII) File',uname='ascii')
save3MID = widget_button(fileMenu,value='Save (2D DAVE) File',uname='s2dave')
exitMID = widget_button(fileMenu,value='Quit',uname='exit',/separator)

; 'Settings' menu
optsID = widget_button(settingsMenu,value='Change Options',uname='opts')

; fill the 'Help' drop-down menu
;aboutMID = widget_button(helpMenu,value='About FANS_DATREDUC',uname='about')
manMID = widget_button(helpMenu,value='Manual',uname='man')

; build main application dialog interface
;voID = widget_label(cb1aID,value='How to treat multi-input files:')
;sumID = widget_button(cb1bID,value='sum',uname='sum',event_pro='dummy')
;apdID = widget_button(cb1bID,value='append',uname='apd',event_pro='dummy')
;sepID = widget_button(cb1bID,value='separate',uname='sep',event_pro='dummy')
;widget_control, sumID, /set_button
;widget_control, sumID, set_uvalue={opt:1, $
;                                   sampStatus:0, $
;                                   bkgdStatus:0, $
;                                   sepSampIndex:0, $
;                                   sepBkgdIndex:0 }

empty='  No files yet  '
voID = widget_label(cb2ID,value='Samples:')
sampID = widget_droplist(cb2ID,value=empty,uname='samp',uvalue=0,event_pro='fans_dropList')
voID = widget_label(cb2ID,value='Bkgd:')
bkgdID = widget_droplist(cb2ID,value=empty,uname='bkgd',event_pro='fans_dropList')
voID = widget_label(cb2ID,value='F. Bkgd:')
fastID = widget_droplist(cb2ID,value=empty,uname='fast',event_pro='fans_dropList')

voID = widget_label(cb3bID,value='Display:')
sViewID = widget_button(cb3cID,value='Sample',uname='samp',uvalue=1,sensitive=0)
bViewID = widget_button(cb3cID,value='Bkgd',uname='bkgd',uvalue=0,sensitive=0)
fViewID = widget_button(cb3cID,value='Fast',uname='fast',uvalue=0,sensitive=0)
oViewID = widget_button(cb3cID,value='Output',uname='outp',uvalue=0,sensitive=0)
widget_control, sViewID,/set_button
fitBkgdID = widget_button(cb3dID,value='Fit Fast',sensitive=0,event_pro='fans_Fastfit')
fitLabID =  widget_label(cb3dID,value='a0 a1 a2:')
parID = widget_text(cb3dID,value='0.00  0.00  0.00 ',/editable,xsize=18,sensitive=0, $
                   event_pro='fans_fastPar')

cbInfo = widget_info(cb3ID,/geometry)
xSize = fix(cbInfo.scr_xsize - 2*cbInfo.xpad)
ySize = fix(0.6*xSize)
plotWinID = widget_draw(cb4ID,xsize=xsize,ysize=ysize,/button_events,event_pro='fans_plotClick')
eSliderID = cw_fslider(cb4ID,title='Energy Transfer Cut (meV)',xsize=xSize,uname='eslider', $
                       min=0.0,max=220.0,format='(F6.2)',uvalue=0)
widget_control, eSliderID, sensitive=0

voID = widget_label(cb4aID,value='Masked detector list:',/dynamic_resize)
vInfo = widget_info(voID,/geometry)
mskListID = widget_text(cb4aID,value='',scr_xsize=0.98*(xSize-vInfo.xsize))

voID = widget_label(cb5aID,value='Output Settings:')
mskOnID = widget_button(cb5bID,value='Apply Mask',uvalue=0,sensitive=0,uname='mskon', $
                        event_pro='fans_outpOpts')
subBkgdID = widget_button(cb5bID,value='Sub Bkgd',uvalue=0,sensitive=0,uname='sbkgd', $
                          event_pro='fans_outpOpts')
subFastID = widget_button(cb5bID,value='Sub Fast Bkgd Fit',uvalue=0,sensitive=0,uname='sfast', $
                          event_pro='fans_outpOpts')

; realise widgets
widget_control, tlbID, /realize

; get the draw window nos and set it as default plot window; also
; desensitize it
widget_control, plotWinID, get_value=plotWinNos,sensitive=0
wset, plotWinNos

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;temporal davePtr definition to test program. davePtr would normally
;be passed down from dave.pro
;commonStr = {instrument:'',$
;             histPtr:ptr_new(/allocate_heap),$
;             xlabel:'',$
;             ylabel:'',$
;             xtype:'',$
;             ytype:'',$
;             xunits: '',$
;             yunits: '',$
;             histLabel:'',$
;             histUnits:'',$
;             treatmentPtr:ptr_new(/allocate_heap),$
;             commonStrPtr:ptr_new()}
;
;dataStr = {commonStr:commonStr,$
;           specificPtr:ptr_new(/allocate_heap)};
;
;daveStr = {dataStrPtr:ptr_new(dataStr)}
;davePtr = ptr_new(daveStr)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; define and set uvalue for the top level base widget
state = {tlbID:tlbID, $
         mfoPtr:ptr_new([2,2,0,1]), $ ; options for multiple files - user configurable 
         nosActDets:50 , $      ; nos of active detectors - user configurable!
         datDir:(*!dave_defaults).datdir, $ ; the data directory (use !dave_defaults)
         workDir:(*!dave_defaults).workdir, $ ; the work directory (use !dave_defaults)
         sampID:sampID, $       ; Sample droplist widget ID
         sampListPtr:ptr_new(/allocate_heap), $ ; ptr to an array of sample files
         sampCount:0, $         ; nos of Sample files
         sampIndex:0, $         ;
         sampSepIndex:0, $
         sampStatus:0, $
         bkgdID:bkgdID, $       ; Bkgd widget ID
         bkgdListPtr:ptr_new(/allocate_heap), $ ; ptr to an array of Bkgd files
         bkgdCount:0, $         ; nos of Bkgd files
         bkgdIndex:0, $         ;
         bkgdSepIndex:0, $
         bkgdStatus:0, $
         fastID:fastID, $       ; Fast bkgd widget ID
         fastListPtr:ptr_new(/allocate_heap), $ ; ptr to a array of Fast bkgd files
         fastCount:0, $         ; nos of Fast bkgd files
         fastIndex:0, $         ;
         fastSepIndex:0, $
         fastStatus:0, $
         outpListPtr:ptr_new(/allocate_heap), $ ; ptr to output files
         outpCount:0, $         ; nos of Fast bkgd files
         outpIndex:0, $         ; 
         doOutpCalc:1, $        ; flag to indicate whether output should be calc.
         sViewID:sViewID, $     ; Disolay Sample widget ID
         bViewID:bViewID, $     ; Display bkgd widget ID
         fViewID:fViewID, $     ; Display Fast bkgd widget ID
         oViewID:oViewID, $     ; Display Output widget ID
         fitBkgdID:fitBkgdID, $ ; fIt bkgd widget ID
         fitLabID:fitLabID, $   ; Fast fit label widget ID
         parID:parID, $         ; fitted Fast bkgd parameters widget ID
         mskListID:mskListID, $ ; Masked detector list widget ID
         mskListPtr:ptr_new(), $ ; ptr to array of detectors to be masked
         updFast:0, $           ; = 1 whenever masked det list changes
         plotWinID:plotWinID, $ ; plot window widget ID
         eSliderID:eSliderID, $ ; Energy transfer cut widget ID
         mskOnID:mskOnID, $     ; Apply Mask widget ID
         subBkgdID:subBkgdID, $ ; sub Bkgd widget ID
         subFastID:subFastID, $ ; sub Fast Bkgd fit widget ID
         sDataPtr:ptr_new(), $  ; ptr to an array of sample data ptrs
         bDataPtr:ptr_new(), $  ; ptr to an array of bkgd data ptrs
         fDataPtr:ptr_new(), $  ; ptr to an array of Fast bkgd data ptrs
         oDataPtr:ptr_new(), $   ; ptr to an array of output data ptrs
         davePtr:davePtr, $
         notifyIds:notifyIds $
        }
sPtr = ptr_new(state,/no_copy)
widget_control, tlbID, set_uvalue=sPtr

xmanager, 'fans_datreduc',tlbID,cleanup='fans_datreduc_cleanup' ;,/no_block

;xmanager,'fans_browser',tlbID,event_handler='resize',cleanup='fans_browser_cleanup'

end
