;$Id: gendast.pro,v 1.1 2002/04/23 14:47:01 jmspark Exp $
pro gendast,data,nsvhn,nnvhn,nden,ndpn,npixn,asd=asd
;------------------------------------------------------------------------
;|									|
;| definition of general data storage structure consisting of   	|
;|	svh  : 	string variables in header at nsvh point		|
;|	       	the LESS important strings should be input with		|
;|		LARGER array-pointer number -- nsvh			| 
;|	nvh  : 	numerical variables in header				|
;|	       	the LESS important numbers should be input with		|
;|		LARGER array-pointer number -- nnvh			| 
;|	 de  : 	data elements measured at ndp data points		|
;|	ndp  :  actual number of data point				|
;|    inorm  :  index for the intensity normalization reference. 	|
;|		(   	e.g. if its monitor rate then 			|
;|		  	data.tnvh(.data.inorm)='mon'			|
;|		 	data.unvh(data.inorm)='count' 		)	|
;|									|
;| each variable has a title and in addition				|
;| each numerical variable has a name and a unit stored			|
;|									|
;| input elements:							|
;|	nsvhn : # of string headers, stored in data.nsvh		|
;|	nnvhn : # of numerical headers, stored in data.nnvh		|
;|      nden  : # of data elements, stored in data.nde			|
;|	ndpn  : # of data points, stored in data.nsvh			|
;|	npixn  : # of pixel points of ASD, stored in data.npix		|
;|									|
;| If the above input elements are not provided, the default values	|
;| of 8,20,8,50 will be used.						|   
;------------------------------------------------------------------------

;----------------------------------------
;| default value of nsvh, nnvh, nde	|
;----------------------------------------
if n_elements(nsvht) eq 0 then nsvht=8 
if n_elements(nnvht) eq 0 then nnvht=20 
if n_elements(ndet) eq 0 then ndet=8 
if n_elements(ndpt) eq 0 then ndpt=50 

;----------------------------------------
;| the data structure			|
;----------------------------------------
 
if keyword_set(asd) then 	$
   data	=	{nsvh:nsvhn,npix:npixn,nnvh:nnvhn,nde:nden,ndp:ndpn	$
		,inorm:-1,tsvh:strarr(nsvhn),svh:strarr(nsvhn)		$
		,tnvh:strarr(nnvhn),unvh:strarr(nnvhn)			$
		,nvh:fltarr(nnvhn),tde:strarr(nden),ude:strarr(nden)	$
		,de:fltarr(ndpn,nden),mon:1.,scan:' '			$
		,asd:fltarr(ndpn,npixn),dasd:fltarr(ndpn,npixn)		$
		,q:fltarr(ndpn,npixn),hw:fltarr(ndpn,npixn)		$
		,qpar:fltarr(ndpn,npixn),qper:fltarr(ndpn,npixn)}	$
else				$
   data	=	{nsvh:nsvhn,nnvh:nnvhn,nde:nden,ndp:ndpn	$
		,inorm:-1,tsvh:strarr(nsvhn),svh:strarr(nsvhn)		$
		,tnvh:strarr(nnvhn),unvh:strarr(nnvhn)			$
		,nvh:fltarr(nnvhn),tde:strarr(nden),ude:strarr(nden)	$
		,de:fltarr(ndpn,nden),mon:1.,scan:' '}

end
